import { languages, CompletionItemKind, MarkdownString, Range, } from 'vscode';
import { parseSignatureHelpFromText } from './docs/parse-signature-help-from-text.js';
const MAX_MATCH_LINES = 30;
export function registerVSCodeLanguageProviders(ctx, language = ['dolphindb', 'dolphindb-python'], docsAnalyser) {
    function wrapMarkdownString(md) {
        return new MarkdownString(md);
    }
    ctx.subscriptions.push(languages.registerCompletionItemProvider(language, {
        provideCompletionItems(doc, pos) {
            const keyword = doc.getText(doc.getWordRangeAtPosition(pos));
            const { functions, constants, keywords } = docsAnalyser.searchCompletionItems(keyword);
            return [
                ...keywords.map((kw) => ({
                    label: kw,
                    insertText: kw,
                    kind: CompletionItemKind.Keyword,
                })),
                ...constants.map((constant) => ({
                    label: constant,
                    insertText: constant,
                    kind: CompletionItemKind.Constant,
                })),
                ...functions.map((fn) => ({
                    label: fn,
                    insertText: fn,
                    kind: CompletionItemKind.Function,
                })),
            ];
        },
        resolveCompletionItem(item, _canceller) {
            const md = docsAnalyser.getFunctionMarkdown(item.label);
            if (md) {
                item.documentation = wrapMarkdownString(md);
            }
            return item;
        },
    }), languages.registerHoverProvider(language, {
        provideHover(doc, pos, _canceller) {
            const word = doc.getText(doc.getWordRangeAtPosition(pos));
            if (!word) {
                return;
            }
            const md = docsAnalyser.getFunctionMarkdown(word);
            if (!md) {
                return;
            }
            return {
                contents: [wrapMarkdownString(md)],
            };
        },
    }), languages.registerSignatureHelpProvider(language, {
        provideSignatureHelp(doc, position, _canceller) {
            const text = doc.getText(new Range(Math.max(position.line - MAX_MATCH_LINES, 0), 0, position.line, position.character));
            const result = parseSignatureHelpFromText(text, docsAnalyser);
            if (!result) {
                return;
            }
            const { activeParameter, documentationMD, signature } = result;
            return {
                activeSignature: 0,
                activeParameter: -1,
                signatures: [
                    {
                        label: signature.fullText,
                        activeParameter: activeParameter,
                        documentation: documentationMD ? wrapMarkdownString(documentationMD) : undefined,
                        parameters: signature.parameters.map((param) => ({
                            label: param.fullText,
                        })),
                    },
                ],
            };
        },
    }, {
        triggerCharacters: ['(', ','],
        retriggerCharacters: [','],
    }));
}
//# sourceMappingURL=vscode.js.map